/// <reference types="react" />
import type { ApolloClientOptions, setLogVerbosity, ApolloCache } from '@apollo/client';
import * as apolloClient from '@apollo/client';
import type { UseAuth } from '@redwoodjs/auth';
import './typeOverride';
import { fragmentRegistry, registerFragment, registerFragments } from './fragmentRegistry';
import { useCache } from './useCache';
export type { CacheKey, FragmentIdentifier, RegisterFragmentResult, } from './fragmentRegistry';
export { useCache };
export { fragmentRegistry, registerFragment, registerFragments };
export type ApolloClientCacheConfig = apolloClient.InMemoryCacheConfig;
export type RedwoodApolloLinkName = 'withToken' | 'authMiddleware' | 'updateDataApolloLink' | 'httpLink';
export type RedwoodApolloLink<Name extends RedwoodApolloLinkName, Link extends apolloClient.ApolloLink = apolloClient.ApolloLink> = {
    name: Name;
    link: Link;
};
export type RedwoodApolloLinks = [
    RedwoodApolloLink<'withToken'>,
    RedwoodApolloLink<'authMiddleware'>,
    RedwoodApolloLink<'updateDataApolloLink'>,
    RedwoodApolloLink<'httpLink', apolloClient.ApolloLink | apolloClient.HttpLink>
];
export type RedwoodApolloLinkFactory = (links: RedwoodApolloLinks) => apolloClient.ApolloLink;
export type GraphQLClientConfigProp = Omit<ApolloClientOptions<unknown>, 'cache' | 'link'> & {
    cache?: ApolloCache<unknown>;
    /**
     * Configuration for Apollo Client's `InMemoryCache`.
     * See https://www.apollographql.com/docs/react/caching/cache-configuration/.
     */
    cacheConfig?: ApolloClientCacheConfig;
    /**
     * Configuration for the terminating `HttpLink`.
     * See https://www.apollographql.com/docs/react/api/link/apollo-link-http/#httplink-constructor-options.
     *
     * For example, you can use this prop to set the credentials policy so that cookies can be sent to other domains:
     *
     * ```js
     * <RedwoodApolloProvider graphQLClientConfig={{
     *   httpLinkConfig: { credentials: 'include' }
     * }}>
     * ```
     */
    httpLinkConfig?: apolloClient.HttpOptions;
    /**
     * Extend or overwrite `RedwoodApolloProvider`'s Apollo Link.
     *
     * To overwrite Redwood's Apollo Link, just provide your own `ApolloLink`.
     *
     * To extend Redwood's Apollo Link, provide a function—it'll get passed an array of Redwood's Apollo Links
     * which are objects with a name and link property:
     *
     * ```js
     * const link = (redwoodApolloLinks) => {
     *   const consoleLink = new ApolloLink((operation, forward) => {
     *     console.log(operation.operationName)
     *     return forward(operation)
     *   })
     *
     *   return ApolloLink.from([consoleLink, ...redwoodApolloLinks.map(({ link }) => link)])
     * }
     * ```
     *
     * If you do this, there's a few things you should keep in mind:
     * - your function should return a single link (e.g., using `ApolloLink.from`; see https://www.apollographql.com/docs/react/api/link/introduction/#additive-composition)
     * - the `HttpLink` should come last (https://www.apollographql.com/docs/react/api/link/introduction/#the-terminating-link)
     */
    link?: apolloClient.ApolloLink | RedwoodApolloLinkFactory;
};
export declare const RedwoodApolloProvider: React.FunctionComponent<{
    graphQLClientConfig?: GraphQLClientConfigProp;
    fragments?: apolloClient.DocumentNode[];
    useAuth?: UseAuth;
    logLevel?: ReturnType<typeof setLogVerbosity>;
    children: React.ReactNode;
}>;
//# sourceMappingURL=index.d.ts.map